#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
   source_generator.py
   author: Ludwig Reiter <ludwig.reiter@intevation.de>

   Read yaml-based form description as generated by
   https://intevation.bitbucket.io/ringoformulare/

   Write out source and layout sections of ringo (formbar) forms.

"""
import codecs
import sys
from xml.etree.ElementTree import Element, SubElement, tostring
import yaml
import argparse

import datetime


#
# Basics
#

def read_stuff(filename):
    with codecs.open(filename, "r", encoding="utf-8") as input_file:
        result = yaml.safe_load(input_file)
    return result

def write_source(xml_string):
    print xml_string



#
# Source
#

def get_type(entity):
    type_string = entity["Typ"]
    if type_string == "Zahl":
        return "integer"
    elif type_string == "Textfeld":
        return "string"
    elif type_string == "Freitext":
        return "string"
    elif type_string == "Datum":
        return "date"
    elif type_string == "Selectbox":
        return "string"
    elif type_string == "Checkbox":
        return "string"
    elif type_string == "Radio":
        return "string"
    else:
        raise ValueError("Found unknown type {}".format(type_string))


def add_required_or_desired(attributes, entity):
    if entity.get("Pflichtfeld") == "Ja":
        attributes["required"] = "true"
    elif entity.get("Forderfeld") == "Ja":
        attributes["desired"] = "true"


def add_renderer_if_needed(sub, entity):
    type1 = None
    if entity["Typ"] == "Selectbox":
        type1 = "dropdown"
    elif entity["Typ"] == "Checkbox":
        type1 = "checkbox"
    elif entity["Typ"] == "Radio":
        type1 = "radio"
    elif entity["Typ"] == "Freitext":
        type1 = "textarea"

    if type1:
        SubElement(sub, "renderer", {"type": type1})


def add_entity(top, entity):
    name = entity["Feldname"]
    label = entity["Text"]
    category = get_type(entity)

    attributes = {"name": name, "id": name, "label": label, "type": category}
    add_required_or_desired(attributes, entity)

    sub = SubElement(top, "entity", attributes)

    if entity.get("Optionen"):
        add_options(sub, [(option["Wert"], option["Text"])
                          for option in entity.get("Optionen")])

    add_renderer_if_needed(sub, entity)

    if entity.get("Anmerkungen"):
        date = datetime.datetime.now().strftime("%Y-%m-%d")
        meta = SubElement(sub, "metadata")
	SubElement(meta, "meta", {"type": "comment", "date": date}).text = \
            entity.get("Anmerkungen")

    return sub


def add_options(sub, option_list):
    options_sub = SubElement(sub, "options")
    for value, label in option_list:
        SubElement(options_sub, "option", {"value": value}).text = label


def generate_the_source(yaml_input):
    top = Element("source")
    for field in yaml_input:
        add_entity(top, field)

    return tostring(top, "UTF-8")



#
# Layout
#

def add_simple_row(top, name):
    sub = SubElement(SubElement(top, "row"), "col")
    field = SubElement(sub, "field", {"ref": name})
    return field

def generate_the_layout(yaml_config):
    top = Element("snippet", {"id": "autogenerated"})
    for entity in yaml_config:
        add_simple_row(top, entity["Feldname"])
    return tostring(top, "UTF-8")



#
# Main
#

def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("filename", help="Path to input file")
    parser.add_argument("-s", "--source", 
                               help="generate source definition (default)",
                               action='store_const',
                               const=True)
    parser.add_argument("-l", "--layout", 
                               help="generate layout definition",
                               action='store_const',
                               const=True)
    parser.add_argument("-c", "--change", 
                               help="change input feldname with text",
                               action='store_const',
                               const=True)
    args = parser.parse_args()


    yaml_config = read_stuff(args.filename)

    if args.change:
        for entity in yaml_config:
            buf = entity['Feldname']
            entity['Feldname'] = entity['Text']
            entity['Text'] = buf

    if args.source or not args.layout:
        xml_string = generate_the_source(yaml_config)
        write_source(xml_string)

    if args.layout:
        xml_string = generate_the_layout(yaml_config)
        write_source(xml_string)

if __name__ == "__main__":
    main()
